from clikit.api.args.format import Argument
from clikit.api.args.format import Option

from cleo import argument
from cleo import option


def test_argument():
    arg = argument("foo", "Foo")

    assert "Foo" == arg.description
    assert arg.is_required()
    assert not arg.is_optional()
    assert not arg.is_multi_valued()
    assert arg.default is None

    arg = argument("foo", "Foo", optional=True, default="bar")

    assert not arg.is_required()
    assert arg.is_optional()
    assert not arg.is_multi_valued()
    assert "bar" == arg.default

    arg = argument("foo", "Foo", multiple=True)

    assert arg.is_required()
    assert not arg.is_optional()
    assert arg.is_multi_valued()
    assert [] == arg.default

    arg = argument("foo", "Foo", optional=True, multiple=True, default=["bar"])

    assert not arg.is_required()
    assert arg.is_optional()
    assert arg.is_multi_valued()
    assert ["bar"] == arg.default


def test_option():
    opt = option("foo", "f", "Foo")

    assert "Foo" == opt.description
    assert not opt.accepts_value()
    assert not opt.is_value_optional()
    assert not opt.is_value_required()
    assert not opt.is_multi_valued()
    assert opt.default is None

    opt = option("foo", "f", "Foo", flag=False)

    assert "Foo" == opt.description
    assert opt.accepts_value()
    assert not opt.is_value_optional()
    assert opt.is_value_required()
    assert not opt.is_multi_valued()

    opt = option("foo", "f", "Foo", flag=False, value_required=False)

    assert "Foo" == opt.description
    assert opt.accepts_value()
    assert opt.is_value_optional()
    assert not opt.is_value_required()
    assert not opt.is_multi_valued()

    opt = option("foo", "f", "Foo", flag=False, multiple=True)

    assert "Foo" == opt.description
    assert opt.accepts_value()
    assert not opt.is_value_optional()
    assert opt.is_value_required()
    assert opt.is_multi_valued()
    assert [] == opt.default

    opt = option("foo", "f", "Foo", flag=False, default="bar")

    assert "Foo" == opt.description
    assert opt.accepts_value()
    assert not opt.is_value_optional()
    assert opt.is_value_required()
    assert not opt.is_multi_valued()
    assert "bar" == opt.default



from cleo.commands.command import Command


class SignatureCommand(Command):

    name = "no:configure"
    signature = (
        "signature:command {foo : Foo} {bar? : Bar} {--z|baz : Baz} {--Z|bazz : Bazz}"
    )

    description = "description"

    help = "help"

    def handle(self):
        self.line("handle called")



# -*- coding: utf-8 -*-

from cleo import Command


class HelloCommand(Command):
    """
    Complete me please.

    hello
        { --dangerous-option= : This $hould be `escaped`. }
        { --option-without-description }
    """



from clikit.api.args.format import Argument
from clikit.api.args.format import Option

from cleo.parser import Parser


def test_basic_parameter_parsing():
    results = Parser.parse("command:name")

    assert "command:name" == results["name"]

    results = Parser.parse("command:name {argument} {--option}")

    assert "command:name" == results["name"]
    assert "argument" == results["arguments"][0].name
    assert "option" == results["options"][0].long_name
    assert Option.NO_VALUE == results["options"][0].flags

    results = Parser.parse("command:name {argument*} {--option=}")

    assert "command:name" == results["name"]
    assert "argument" == results["arguments"][0].name
    assert Argument.MULTI_VALUED | Argument.REQUIRED == results["arguments"][0].flags
    assert "option" == results["options"][0].long_name
    assert Option.REQUIRED_VALUE == results["options"][0].flags

    results = Parser.parse("command:name {argument?*} {--option=*}")

    assert "command:name" == results["name"]
    assert "argument" == results["arguments"][0].name
    assert Argument.MULTI_VALUED | Argument.OPTIONAL == results["arguments"][0].flags
    assert "option" == results["options"][0].long_name
    assert Option.MULTI_VALUED == results["options"][0].flags

    results = Parser.parse(
        "command:name {argument?* : The argument description.}    {--option=* : The option description.}"
    )

    assert "command:name" == results["name"]
    assert "argument" == results["arguments"][0].name
    assert "The argument description." == results["arguments"][0].description
    assert Argument.MULTI_VALUED | Argument.OPTIONAL == results["arguments"][0].flags
    assert "option" == results["options"][0].long_name
    assert "The option description." == results["options"][0].description
    assert Option.MULTI_VALUED == results["options"][0].flags

    results = Parser.parse(
        "command:name "
        "{argument?* : The argument description.}    "
        "{--option=* : The option description.}"
    )

    assert "command:name" == results["name"]
    assert "argument" == results["arguments"][0].name
    assert "The argument description." == results["arguments"][0].description
    assert Argument.MULTI_VALUED | Argument.OPTIONAL == results["arguments"][0].flags
    assert "option" == results["options"][0].long_name
    assert "The option description." == results["options"][0].description
    assert Option.MULTI_VALUED == results["options"][0].flags


def test_shortcut_name_parsing():
    results = Parser.parse("command:name {--o|option}")

    assert "command:name" == results["name"]
    assert "option" == results["options"][0].long_name
    assert "o" == results["options"][0].short_name
    assert Option.NO_VALUE == results["options"][0].flags

    results = Parser.parse("command:name {--o|option=}")

    assert "command:name" == results["name"]
    assert "option" == results["options"][0].long_name
    assert "o" == results["options"][0].short_name
    assert Option.REQUIRED_VALUE == results["options"][0].flags

    results = Parser.parse("command:name {--o|option=*}")

    assert "command:name" == results["name"]
    assert "option" == results["options"][0].long_name
    assert "o" == results["options"][0].short_name
    assert Option.MULTI_VALUED == results["options"][0].flags

    results = Parser.parse("command:name {--o|option=* : The option description.}")

    assert "command:name" == results["name"]
    assert "option" == results["options"][0].long_name
    assert "o" == results["options"][0].short_name
    assert "The option description." == results["options"][0].description
    assert Option.MULTI_VALUED == results["options"][0].flags

    results = Parser.parse("command:name " "{--o|option=* : The option description.}")

    assert "command:name" == results["name"]
    assert "option" == results["options"][0].long_name
    assert "o" == results["options"][0].short_name
    assert "The option description." == results["options"][0].description
    assert Option.MULTI_VALUED == results["options"][0].flags



import os
import pytest

from cleo.commands import Command
from cleo.application import Application
from cleo.testers.application_tester import ApplicationTester


class FooCommand(Command):
    """
    Foo command

    foo
        {foo : Foo argument}
    """

    def handle(self):
        self.line(self.argument("foo"))


@pytest.fixture()
def app():
    app = Application()
    app.config.set_terminate_after_run(False)
    app.add(FooCommand())

    return app


@pytest.fixture()
def tester(app):
    return ApplicationTester(app)


def test_execute(tester):
    assert 0 == tester.execute("foo bar")
    assert 0 == tester.status_code
    assert "bar\n" == tester.io.fetch_output()



# -*- coding: utf-8 -*-

import os
import pytest

from cleo import Application
from cleo import CommandTester

from .fixtures.hello_command import HelloCommand
from .fixtures.command_with_colons import CommandWithColons

app = Application()
app.add(HelloCommand())
app.add(CommandWithColons())


def test_invalid_shell():
    command = app.find("completions")
    tester = CommandTester(command)

    with pytest.raises(ValueError):
        tester.execute("pomodoro")


def test_bash(mocker):
    mocker.patch(
        "clikit.api.args.args.Args.script_name",
        new_callable=mocker.PropertyMock,
        return_value="/path/to/my/script",
    )
    mocker.patch(
        "cleo.commands.completions_command.CompletionsCommand._generate_function_name",
        return_value="_my_function",
    )

    command = app.find("completions")
    tester = CommandTester(command)
    tester.execute("bash")

    with open(os.path.join(os.path.dirname(__file__), "fixtures", "bash.txt")) as f:
        expected = f.read()

    assert expected == tester.io.fetch_output().replace("\r\n", "\n")


def test_zsh(mocker):
    mocker.patch(
        "clikit.api.args.args.Args.script_name",
        new_callable=mocker.PropertyMock,
        return_value="/path/to/my/script",
    )
    mocker.patch(
        "cleo.commands.completions_command.CompletionsCommand._generate_function_name",
        return_value="_my_function",
    )

    command = app.find("completions")
    tester = CommandTester(command)
    tester.execute("zsh")

    with open(os.path.join(os.path.dirname(__file__), "fixtures", "zsh.txt")) as f:
        expected = f.read()

    assert expected == tester.io.fetch_output().replace("\r\n", "\n")


def test_fish(mocker):
    mocker.patch(
        "clikit.api.args.args.Args.script_name",
        new_callable=mocker.PropertyMock,
        return_value="/path/to/my/script",
    )
    mocker.patch(
        "cleo.commands.completions_command.CompletionsCommand._generate_function_name",
        return_value="_my_function",
    )

    command = app.find("completions")
    tester = CommandTester(command)
    tester.execute("fish")

    with open(os.path.join(os.path.dirname(__file__), "fixtures", "fish.txt")) as f:
        expected = f.read()

    assert expected == tester.io.fetch_output().replace("\r\n", "\n")



# -*- coding: utf-8 -*-
from cleo.application import Application
from cleo.commands.command import Command
from cleo.helpers import argument
from cleo.testers import CommandTester

from ..fixtures.signature_command import SignatureCommand
from ..fixtures.inherited_command import ChildCommand


class MyCommand(Command):
    """
    Command testing.

    test
        {action : The action to execute.}
    """

    def handle(self):
        action = self.argument("action")

        getattr(self, "_" + action)()

    def _overwrite(self):
        self.write("Processing...")
        self.overwrite("Done!")


class MySecondCommand(Command):

    name = "test2"
    description = "Command testing"

    arguments = [argument("foo", "Bar", multiple=True)]

    def handle(self):
        foos = self.argument("foo")

        self.line(",".join(foos))


def test_set_application():
    application = Application()
    command = Command()
    command.set_application(application)

    assert application == command.application


def test_with_signature():
    command = SignatureCommand()
    config = command.config

    assert "signature:command" == config.name
    assert "description" == config.description
    assert "help" == config.help
    assert 2 == len(config.arguments)
    assert 2 == len(config.options)


def test_signature_inheritance():
    command = ChildCommand()
    config = command.config

    assert "parent" == config.name
    assert "Parent Command." == config.description


def test_overwrite():
    command = MyCommand()

    tester = CommandTester(command)
    tester.execute("overwrite", decorated=True)

    expected = "Processing...{}Done!        {}".format("\x08" * 13, "\x08" * 8)
    assert expected == tester.io.fetch_output()


def test_explicit_multiple_argument():
    command = MySecondCommand()

    tester = CommandTester(command)
    tester.execute("1 2 3")

    assert "1,2,3\n" == tester.io.fetch_output()



import os

import pytest

from cleo.commands.command import Command
from cleo.testers.command_tester import CommandTester


class FooCommand(Command):
    """
    Foo command

    foo
        {foo : Foo argument}
    """

    def handle(self):
        self.line(self.argument("foo"))


@pytest.fixture()
def tester():
    return CommandTester(FooCommand())


def test_execute(tester):
    assert 0 == tester.execute("bar")
    assert 0 == tester.status_code
    assert "bar\n" == tester.io.fetch_output()
